/******************************************************************************
 *
 * Copyright (C) 2011 Harri Hyttinen
 * Copyright (C) 2011 Juuso Korhonen
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 *
 *****************************************************************************/
 
 
import QtQuick 1.1
import com.nokia.meego 1.0

Sheet {
    id: editSheet
    acceptButtonText: "Save"
    rejectButtonText: "Cancel"

    signal saved(int index, string title, string album, string artist)

    property string songFileName: ""
    property string mimeType: ""
    property string title: ""
    property bool editFocused: false
    property bool saveClicked: false
    property int index: -1

    onEditFocusedChanged: {
        var hide = (!appWindow.inPortrait && editSheet.editFocused)
        infoLabel.setHidden(hide)
    }

    function showSaveResult(success) {
        if(success) {
            appWindow.showBanner("Saved", mainPage)
        } else {
            appWindow.showBanner("Save failed", mainPage)
        }
    }

    TagEditor {
        id: tageditor
        property bool saveFinished: false
        property bool saveResult: false
        onArtAvailable: {
            albumArt.source = "image://albumArt/" + tageditor.album;
        }

        onSaveFinished: {
            saveFinished = true
            saveResult = success
            if(!saveIndicatorHideTimer.running) {
                handleSaveFinished()
            }
        }
    }

    function handleSaveFinished() {
        saveProgressIndicator.visible = false
        saved(index, tageditor.title, tageditor.album, tageditor.artist)
        editSheet.close()
    }

    Timer {
        id: saveIndicatorHideTimer
        interval: 600
        running: false
        onTriggered: {
            if(tageditor.saveFinished) {
                handleSaveFinished()
            }
        }
    }

    Rectangle {
        id: saveProgressIndicator
        anchors.fill: parent
        visible: false
        Behavior on color { PropertyAnimation {} }

        MouseArea {
            anchors.fill: parent
        }

        BusyIndicator {
            id: indicator
            platformStyle: BusyIndicatorStyle { size: "large" }
            running: false
            anchors.centerIn: parent
            z: 2
            onVisibleChanged: {
                if(visible) {
                    running = true
                    saveIndicatorHideTimer.start()
                    saveProgressIndicator.color = "#DCDCDCDC"
                } else {
                    running = false
                    saveProgressIndicator.color = "#00DCDCDC"
                }
            }
        }
    }

    Component.onCompleted: {
        appWindow.orientationChangeFinished.connect(orientationChanged)
    }

    function orientationChanged() {
        if(!appWindow.inPortrait) {
            infoLabel.setHidden(editSheet.editFocused)
        } else {
            infoLabel.setHidden(false)
        }
    }

    Gradient {
        id: labelFlickableAlphaGradient
        GradientStop { position: 0.0; color: "#FFBEBEBE" }
        GradientStop { position: 1.0; color: "#00000000" }
    }

    content:
        Rectangle {
        id: editSheetBackground
        anchors.fill: parent
        color: "#DCDCDC"

        Rectangle {
            id: infoLabel
            y: parent.y
            width: parent.width
            height: infoLabelTextColumn.height + 20
            color: "#DCDCDC"
            z: 1

            PropertyAnimation {
                id: showAnimation
                target: infoLabel
                property: "y"
                to: editSheetBackground.y + editSheet.content
                duration: 300
                easing.type: Easing.OutQuad;
            }

            function setHidden(value) {
                if(value) {
                    showAnimation.stop()
                    showAnimation.to = editSheetBackground.y-infoLabel.height
                    showAnimation.running = true
                } else {
                    showAnimation.stop()
                    showAnimation.to = editSheetBackground.y
                    showAnimation.running = true
                }
            }

            Image {
                id: albumArt
                anchors.verticalCenter: infoLabel.verticalCenter
                anchors.left: parent.left
                anchors.leftMargin: 10
                height: infoLabel.height - 10
                width: height
                source: "n9tagedit_noart.png"
            }

            Column {
                id: infoLabelTextColumn
                anchors.verticalCenter: parent.verticalCenter
                anchors.left: albumArt.right
                anchors.leftMargin: 30

                Text {
                    id: infoLabelDuration
                    font.pointSize: 14
                    font.family: "Nokia Pure Text"
                    text: "Duration: " + getTime()

                    function getTime() {
                        if(Qt.formatTime(tageditor.length, "h") == "0") {
                            return Qt.formatTime(tageditor.length, "m 'min' ss 'sec'")
                        } else {
                            return Qt.formatTime(tageditor.length, "h 'h' mm 'min' ss 'sec'")
                        }
                        return ""
                    }
                }

                Text {
                    id: infoLabelBitrate
                    font.pointSize: 14
                    font.family: "Nokia Pure Text"
                    text: "Bitrate: " + tageditor.bitrate + " kbit/s"
                }

                Text {
                    id: infoLabelSampleRate
                    font.pointSize: 14
                    font.family: "Nokia Pure Text"
                    text: "Samplerate: " + tageditor.sampleRate/1000 + " kHz"
                }

                Text {
                    id: infoMimeType
                    font.pointSize: 14
                    font.family: "Nokia Pure Text"
                    text: "Media type: " + mimeType
                }
            }

            MouseArea {
                anchors.fill: parent
                onClicked: {
                    // \todo play?
                }
            }
        }

        Rectangle {
            id: titleShadow
            anchors.top: infoLabel.bottom;
            width: parent.width
            height: 10
            gradient: labelFlickableAlphaGradient
            z: 1
        }

        Flickable {
            id: flickableContent
            anchors.top: infoLabel.bottom
            anchors.bottom: parent.bottom
            width: parent.width
            // \note remember to change this when adding items!
            contentHeight: titleItem.height + albumItem.height +
                           artistItem.height + genreItem.height +
                           yearItem.height + commentItem.height

            contentWidth: parent.width
            clip: true

            SheetItem {
                anchors.top: parent.top
                id: titleItem
                labelText: "Title:"
                editText: tageditor.title.length ? tageditor.title : editSheet.title
                onFocusedChanged: {
                    editSheet.editFocused = focused
                }
            }

            SheetItem {
                anchors.top: titleItem.bottom
                id: albumItem
                labelText: "Album:"
                editText: tageditor.album
                onFocusedChanged: {
                    editSheet.editFocused = focused
                }
            }

            SheetItem {
                anchors.top: albumItem.bottom
                id: artistItem
                labelText: "Artist:"
                editText: tageditor.artist
                onFocusedChanged: {
                    editSheet.editFocused = focused
                }
            }

            SelectionDialog {
                 id: genreSelectionDialog
                 titleText: "Genre selection"
                 anchors.fill: parent
                 selectedIndex: -1

                 model: ListModel {
                     ListElement { name: "Acid" }
                     ListElement { name: "Acid Jazz" }
                     ListElement { name: "Acid Punk" }
                     ListElement { name: "Alternative" }
                     ListElement { name: "AlternRock" }
                     ListElement { name: "Ambient" }
                     ListElement { name: "Audiobook" } //not an id3 genre
                     ListElement { name: "Bass" }
                     ListElement { name: "Blues" }
                     ListElement { name: "Cabaret" }
                     ListElement { name: "Christian Rap" }
                     ListElement { name: "Classic Rock" }
                     ListElement { name: "Classical" }
                     ListElement { name: "Comedy" }
                     ListElement { name: "Country" }
                     ListElement { name: "Cult" }
                     ListElement { name: "Dance" }
                     ListElement { name: "Darkwave" }
                     ListElement { name: "Death Metal" }
                     ListElement { name: "Disco" }
                     ListElement { name: "Dream" }
                     ListElement { name: "Electronic" }
                     ListElement { name: "Ethnic" }
                     ListElement { name: "Euro-Techno" }
                     ListElement { name: "Eurodance" }
                     ListElement { name: "Funk" }
                     ListElement { name: "Fusion" }
                     ListElement { name: "Game" }
                     ListElement { name: "Gangsta" }
                     ListElement { name: "Gospel" }
                     ListElement { name: "Gothic" }
                     ListElement { name: "Grunge" }
                     ListElement { name: "Hard Rock" }
                     ListElement { name: "Hip-Hop" }
                     ListElement { name: "House" }
                     ListElement { name: "Industrial" }
                     ListElement { name: "Instrumental" }
                     ListElement { name: "Instrumental Pop" }
                     ListElement { name: "Instrumental Rock" }
                     ListElement { name: "Jazz" }
                     ListElement { name: "Jazz+Funk" }
                     ListElement { name: "Jungle" }
                     ListElement { name: "Lo-Fi" }
                     ListElement { name: "Meditative" }
                     ListElement { name: "Metal" }
                     ListElement { name: "Musical" }
                     ListElement { name: "Native American" }
                     ListElement { name: "New Age" }
                     ListElement { name: "New Wave" }
                     ListElement { name: "Noise" }
                     ListElement { name: "Oldies" }
                     ListElement { name: "Other" }
                     ListElement { name: "Podcast" } //not an id3 genre
                     ListElement { name: "Polka" }
                     ListElement { name: "Pop" }
                     ListElement { name: "Pop-Folk" }
                     ListElement { name: "Pop/Funk" }
                     ListElement { name: "Pranks" }
                     ListElement { name: "Psychadelic" }
                     ListElement { name: "Punk" }
                     ListElement { name: "R&B" }
                     ListElement { name: "Rap" }
                     ListElement { name: "Rave" }
                     ListElement { name: "Reggae" }
                     ListElement { name: "Retro" }
                     ListElement { name: "Rock" }
                     ListElement { name: "Rock & Roll" }
                     ListElement { name: "Showtunes" }
                     ListElement { name: "Ska" }
                     ListElement { name: "Soul" }
                     ListElement { name: "Sound Clip" }
                     ListElement { name: "Soundtrack" }
                     ListElement { name: "Southern Rock" }
                     ListElement { name: "Space" }
                     ListElement { name: "Techno" }
                     ListElement { name: "Techno-Industrial" }
                     ListElement { name: "Top 40" }
                     ListElement { name: "Trailer" }
                     ListElement { name: "Trance" }
                     ListElement { name: "Tribal" }
                     ListElement { name: "Trip-Hop" }
                     ListElement { name: "Vocal" }
                 }
                 onAccepted: {
                     tageditor.genre = model.get(selectedIndex).name
                     selectedIndex = -1
                 }
             }

            SheetItem {
                anchors.top: artistItem.bottom
                id: genreItem
                labelText: "Genre:"
                editText: tageditor.genre
                onFocusedChanged: {
                    editSheet.editFocused = focused
                }
                ToolIcon {
                    id: comboboxArrowIcon
                    anchors.top: parent.editField.top
                    anchors.right: parent.editField.right
                    iconId: "common-combobox-arrow"
                    height: parent.editField.height
                    width: parent.editField.height
                    onClicked: {
                        genreSelectionDialog.open()
                    }
                }
            }

            SheetItem {
                anchors.top: genreItem.bottom
                oneRow: true
                id: yearItem
                anchors.left: parent.left
                width: 200
                labelText: "Year:"
                editText: tageditor.year
                inputHints: Qt.ImhDigitsOnly
                onFocusedChanged: {
                    editSheet.editFocused = focused
                }
            }

            SheetItem {
                anchors.top: genreItem.bottom
                oneRow: true
                anchors.left: yearItem.right
                anchors.right: parent.right
                id: trackItem
                labelText: "Track number:"
                inputHints: Qt.ImhDigitsOnly
                editText: tageditor.track
                onFocusedChanged: {
                    editSheet.editFocused = focused
                }
            }

            Rectangle {
                id: commentItem
                anchors.left: parent.left
                anchors.right: parent.right
                anchors.top: trackItem.bottom
                height: commentEdit.height + commentLabel.height
                gradient: trackItem.itemGradient

                Text {
                    id: commentLabel
                    anchors.left: parent.left
                    anchors.leftMargin: 10
                    anchors.top: parent.top
                    text: "Comment:"
                    font.pointSize: 20
                    font.family: "Nokia Pure Text Light"
                    color: "steelblue"
                }

                TextArea {
                    id: commentEdit
                    anchors.top: commentLabel.bottom
                    anchors.left: parent.left
                    anchors.leftMargin: 10
                    anchors.right: parent.right
                    anchors.rightMargin: 10
                    text: tageditor.comment
                    font.pointSize: 18
                    font.family: "Nokia Pure Text"
                    onActiveFocusChanged: {
                        editSheet.editFocused = activeFocus
                    }
                }

                function getText() {
                    return commentEdit.text
                }
            }
        }
    }

    onAccepted: {
        //set edited data to tageditor for saving
        open()
        saveProgressIndicator.visible = true
        tageditor.title = titleItem.getText()
        tageditor.album = albumItem.getText()
        tageditor.artist = artistItem.getText()
        tageditor.genre = genreItem.getText()
        tageditor.year = Number(yearItem.getText().valueOf())
        tageditor.track = Number(trackItem.getText().valueOf())
        tageditor.comment = commentItem.getText()
        saveClicked = true
        tageditor.save()
        songFileName = ""
    }

    onRejected: {
        songFileName = ""
        mimeType = ""
    }

    onStatusChanged: {
        if(status == DialogStatus.Closed) {
            albumArt.source = "n9tagedit_noart.png"
            if(saveClicked) {
                showSaveResult(tageditor.saveResult)
            }
            saveClicked = false
            tageditor.saveFinished = false
            tageditor.saveResult = false
        }
    }

    function initSheet(mime, file, index, title) {
        editSheet.mimeType = mime
        editSheet.songFileName = file
        editSheet.index = index
        editSheet.title = title
        return tageditor.setFile(songFileName, mimeType)
    }
}
